=encoding utf8

=head1 Name

SPVM::Document::ExchangeAPI - SPVM Exchange API

=head1 Description

This document is moved into the document of L<SPVM> and L<SPVM::ExchangeAPI>.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
