package SPVM::Address;

1;

=head1 Name

SPVM::Address - Address

=head1 Description

C<SPVM::Address> is a L<SPVM> class to have a address.

=head1 Usage

  use Address;
  
  my $address = Address->new;

=head1 Class Methods

=head2 new

  static method new : Address ();

Creates a new L<Address|SPVM::Address> object. The address is C<NULL> of the C language.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
