=encoding utf8

=head1 Name

SPVM::Document::LanguageSpecification - SPVM Language Specification

=head1 Description

C<SPVM::Document::LanguageSpecification> is renamed to L<SPVM::Document::Language>.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
