package SPVM::Error;

1;

=head1 Name

SPVM::Error - Error

=head1 Usage

  use Error;

=head1 Description

C<Error> is the error class for general errors. The class id is 1.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
